function [c] = CostKurtR(Y, f)

% [c] = CostKurtR(Y, f)
%
% Estimation of the kurtosis based contrast (real valued signals)
% CostKurtR estimates the value of the contrast (Cum4{r})^2/E{r^2}^2
% where r = f'*Y;
%
% Inputs:
% -------
% Y = (No*D x Nsample) block-toeplitz matrix in which delayed version of
%   the observations are stacked (No: number of observations, D: lenght of
%   the separating filter)
% f = (1 x No*D) row vector containing the coefficients of the separating
%   filter (No: number of observations, D: lenght of the separating filter)
%
% Output: 
% -------
% c = value of the contrast
%
% -------
% Author: Marc Castella
% Created: ??
% Last modified: 04/07/2006
% Contact: marc.castella@int-evry.fr


% Recuperation des dimensions
T = size(Y,2);

% calcul du signal "extrait" pour deflation
r = f'*Y;

% termes intermediaires
D1 = 1/T*sum(r.^4);
D2 = 1/T*sum(r.^2);

% Calcul du cout
c = (D1/D2^2 -3 )^2;
