
function [R,C] = makeRC(x,z,D);

% [R,C] = makeRC(x,z,D);
%
% Estimation of the covariances and cumulants for the optimization of the
%   contrast in [1]
% makeRC estimates the matrices R_x and C_{x,z} described in Eq. (31)-(33)
%   of [1]
%
% Inputs:
% -------
% x = (Nx x Nsample) matrix containing the Nx observations of the mixture
% z = (1 x Nsample) vector containing the reference signal
% D = length of the separating filter
% 
% Outputs: 
% -------
% R = (Nx x Nx) covariance matrix
% C = (Nx x Nx) cumulant matrix
%
% -------
% Author: Marc Castella
% Created: 11/2003
% Last modified: 03/07/2006
% Contact: marc.castella@int-evry.fr
%
% Ref:
% [1] Quadratic Higher-Order Criteria for Iterative Blind Separation of a 
% MIMO Convolutive Mixture of Sources / M. Castella, S. Rhioui, E. Moreau 
% and J.-C. Pesquet, IEEE Trans. on Signal Processing (accepted, to appear)


if (size(x,2)-D+1)~=size(z,2),
    error('xtilde et z n''ont pas meme longueur');
else Ne = size(x,2);
end;

Nx = size(x,1);

Z = repmat(z,Nx*D,1);

xtilde = [];
for d = 1:D,
    xtilde = [ x(:,d:end+d-D); xtilde];
end;

Zxtilde = Z.*xtilde;
Z_xtilde = conj(Z).*xtilde;

% C = 1/Ne*Zxtilde*Zxtilde' ...
%     -1/Ne*(xtilde*xtilde').*(1/Ne*(z*z')) ...
%     -1/Ne*sum(Zxtilde,1)*1/Ne*(sum(Zxtilde,1))' ...
%     -1/Ne*sum(Z_xtilde,1)*1/Ne*(sum(Z_xtilde,1))';

C = 1/Ne*Zxtilde*Zxtilde' ...
    -1/Ne^2*(xtilde*xtilde')*(z*z') ...
    -1/Ne^2*(xtilde*z.')*(xtilde*z.')' ...
    -1/Ne^2*(xtilde*z')*(xtilde*z')';

R = cov(xtilde');
